/*----------------------------------------------------------------------------
 *
 *  ecurses.c - a prototype of a new curses implementation
 *
 *  Location: /usr/local/src/ecurses/ecurses.c
 *
 *  Directory:
 *
 *  addch (ch)    - simple addch function
 *  getch ()      - getch function for terminal input
 *  keyname (ch)  - return name of a pressed key
 *  initscr ()    - simple initscr function with raw-mode function
 *  endwin ()     - restore tty
 *
 *  Author: Frank Meyer
 *----------------------------------------------------------------------------
 */

#include <sys/termio.h>
#include <ecurses.h>
#include <map-table.h>

struct termio oldmode;
struct termio newmode;

extern char * getenv ();


addch (ch)
int ch;
{
  static int  last_chset_no = -1;            /* last charset     */
  int         chset_no;                      /* current charset  */
  int         char_to_send;                  /* char to send     */

  chset_no     = map_table[ch].chset_no;
  char_to_send = map_table[ch].ch_out;

  if (last_chset_no != chset_no)             /* charset changed? */
  {                                          /* yes, send new    */
    last_chset_no = chset_no;                /* charset sequence */
    fputs (charset[chset_no], stdout);
  }
  putc (char_to_send, stdout);               /* send character   */

  return (OK);
} /* addch (ch) */


int
getch ()
{
  int ch;                             /* character to be read     */
  int len = 0;                        /* length of input sequence */
  int scan = 0;                       /* scancode                 */
  unsigned char buf[MAX_KEY_SEQ_LEN]; /* input buffer             */

  fflush (stdout);                    /* 1st flush output stream  */

  ch = getchar ();                    /* read one character       */

  if (key_table[ch][0] == ch && key_table[ch][1] == '\0')
  {                                   /* simple character, return */
    return (ch);                      /* it                       */
  }

  buf[len++] = ch;                    /* no simple character,     */
  buf[len] = '\0';                    /* store it in input buffer */

  while (scan < MAX_KEYS)                /* search input sequence */
  {                                      /* in key table          */
    if (! strncmp (buf, key_table[scan], len))
    {                                    /* found ?               */
      if (key_table[scan][len] == '\0')  /* yes, but more chars?  */
      {                                  /* no, we are ready      */
        return (scan);                   /* return scan code      */
      }
      else                               /* there must be more .. */
      {
        ch = getchar ();                 /* read next character   */
        buf[len++] = ch;                 /* and store it in buf   */
        buf[len] = '\0';
      }
    }
    else                                 /* key_table doesn't     */
    {                                    /* match, try next scan  */
      scan++;                            /* code                  */
    }
  }

  return (KEY_ERR);                      /* unknown key !         */
} /* getch () */


char *
keyname (ch)
int ch;
{
  static char buf[32];                      /* name buffer      */

  if (ch >= 0 && ch < KEY_F0)               /* function key?    */
  {                                         /* no, return       */
    return (keyname_table[ch]);             /* name from table  */
  }
  else if (ch >= KEY_F0 && ch <= KEY_F(31))    /* NORMAL        */
  {                                            /* fkey, return  */
    sprintf (buf, "KEY_F%d", ch - KEY_F0);     /* KEY_Fxx       */
  }
  else if (ch >= KEY_SF0 && ch <= KEY_SF(31))  /* SHIFT         */
  {                                            /* fkey, return  */
    sprintf (buf, "KEY_SF%d", ch - KEY_SF0);   /* KEY_SFxx      */
  }
  else if (ch >= KEY_CF0 && ch <= KEY_CF(31))  /* CTRL          */
  {                                            /* fkey, return  */
    sprintf (buf, "KEY_CF%d", ch - KEY_CF0);   /* KEY_CFxx      */
  }
  else if (ch >= KEY_AF0 && ch <= KEY_AF(31))  /* ALT           */
  {                                            /* fkey, return  */
    sprintf (buf, "KEY_AF%d", ch - KEY_AF0);   /* KEY_AFxx      */
  }
  else if (ch >= KEY_SCF0 && ch <= KEY_SCF(31))/* SHIFT + CTRL  */
  {                                            /* fkey, return  */
    sprintf (buf, "KEY_SCF%d", ch - KEY_SCF0); /* KEY_SCFxx     */
  }
  else if (ch >= KEY_SAF0 && ch <= KEY_SAF(31))/* SHIFT + ALT   */
  {                                            /* fkey, return  */
    sprintf (buf, "KEY_SAF%d", ch - KEY_SAF0); /* KEY_SAFxx     */
  }
  else                                         /* unknown scan  */
  {
    strcpy (buf, "KEY_UNKNOWN");
  }
  return (buf);                                /* return buffer */
} /* keyname (ch) */


int
initscr ()
{
  char * term;                                  /* type of term  */
  char fname[64];                               /* file name buf */
  FILE * fp;                                    /* file pointer  */

  if (! (term = getenv ("TERM")))               /* no environment*/
  {                                             /* var TERM      */
    fprintf (stderr, "ecurses: don't know your terminal name\n");
    exit (1);
  }

  sprintf (fname, "/usr/local/lib/term/iso8/%s.OUT", term);
                                                /* fill fname      */

  if (! (fp = fopen (fname, "r")))              /* open ok ?       */
  {                                             /* no !            */
    fprintf (stderr, "ecurses: cannot open database file %s\n",
             fname);
    exit (1);
  }

  fread (charset, sizeof (charset), 1, fp);     /* read charset    */
  fread (map_table, sizeof (map_table), 1, fp); /* read map_table  */
  fread (key_table, sizeof (key_table), 1, fp); /* read key_table  */

  (void) fclose (fp);                           /* close the file  */

  (void) ioctl (0, TCGETA, &oldmode);
  (void) ioctl (0, TCGETA, &newmode);

  newmode.c_iflag        |= ICRNL;      /* map CR into NL          */
  newmode.c_iflag        &= ~INLCR;     /* don't map NL into CR    */
  newmode.c_cc[VINTR]    = '\377';      /* disable VINTR VQUIT     */
  newmode.c_cc[VQUIT]    = '\377';      /* but don't touch VSWTCH  */
  newmode.c_lflag        &= ~ECHO;      /* echo off                */
  newmode.c_lflag        &= ~ICANON;    /* no canonical input      */
  newmode.c_oflag        &= ~TAB3;      /* switch off TAB conv     */
  newmode.c_cc[VMIN]     = 1;           /* read only one character */
  newmode.c_cc[VTIME]    = 0;           /* block input             */

  (void) ioctl (0, TCSETAW, &newmode);  /* set new mode ...        */

  return (OK);
} /* initscr () */


int
endwin ()
{
    (void) ioctl (0, TCSETAW, &oldmode); /* return to initial mode */
    return (OK);
} /* endwin () */
